//--------------------------------------------------------------//
// Null
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Null";
   string Category    = "Null";
> = 0;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture Input;
sampler InputSampler = sampler_state { Texture = <Input>; };
sampler InputSamplerPoint = sampler_state
 { Texture = <Input>;
   MINFILTER = POINT;
   MIPFILTER = POINT;
   MAGFILTER = POINT;
 };

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 null_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   return tex2D( InputSampler, xy1 );
}

float4 revComp_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSampler, xy1 );

   float4 ret;
   ret.a = src.b;
   ret.r = src.g;
   ret.g = src.r;
   ret.b = src.a;

   return ret;
}

float4 flip_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   return tex2D( InputSampler, float2( xy1.x, 1.0 - xy1.y ) );
}

float4 revAndFlip_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSampler, float2( xy1.x, 1.0 - xy1.y ) );

   float4 ret;
   ret.r = src.a;
   ret.g = src.b;
   ret.b = src.g;
   ret.a = src.r;

   return ret;
}

float4 UYVYtoYUYV_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSampler, xy1 );

   float4 ret;
   ret.r = src.a;
   ret.g = src.b;
   ret.b = src.g;
   ret.a = src.r;

   return ret;
}

float4 YVYUtoYUYV_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSampler, xy1 );

   float4 ret;
   ret.r = src.r;
   ret.g = src.a;
   ret.b = src.b;
   ret.a = src.g;

   return ret;
}

bool isEven( float f )
{
  return ( ( 2 * floor( f / 2 ) ) == f );
}

float _Width = 1280;
float _Height;

float4 YV12toYUYV_main( float2 xyNorm : TEXCOORD0, float2 xy1 : TEXCOORD1 ) : COLOR
{
   float x = floor( ( xyNorm.x * _Width ) + 0.5 );
   float y = floor( ( xyNorm.y * _Height ) + 0.5 );

   bool xEven = isEven( x );
   bool yEven = isEven( y );

   float yX = floor( x / 2 );

   if ( !yEven )
      yX += _Width/2;

   yX /= _Width;

   float uvX = ( xyNorm.x / 4 ) + ( fmod( y,4 ) * 0.25 );

   float4 yPixel = tex2D( InputSampler, float2( yX, xy1.y / 2 ) );
   float4 uPixel = tex2D( InputSampler, float2( uvX, 0.5  + ( xy1.y / 4 ) ) );
   float4 vPixel = tex2D( InputSampler, float2( uvX, 0.75 + ( xy1.y / 4 ) ) );

   float4 yuyv;

   if ( xEven )
   {
      yuyv[0] = yPixel[1];
      yuyv[2] = yPixel[2];
   }
   else
   {
      yuyv[0] = yPixel[3];
      yuyv[2] = yPixel[0];
   }

   int uvIdx = fmod( x, 4 );
   yuyv[1] = uPixel[ uvIdx ];
   yuyv[3] = vPixel[ uvIdx ];

   return yuyv;
}

float4 RotateLeft_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSampler, xy1 );

   float4 ret;
   ret.r = src.a;
   ret.g = src.r;
   ret.b = src.g;
   ret.a = src.b;

   return ret;
}

float4 RotateRight_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSampler, xy1 );

   float4 ret;
   ret.r = src.g;
   ret.g = src.b;
   ret.b = src.a;
   ret.a = src.r;

   return ret;
}

float4 BGRtoARGB_main( float2 xy : TEXCOORD1, float2 xyl : TEXCOORD2 ) : COLOR
{
   float4 ret;

   float2 xyr;

   // shift a quarter pixel right
   xyr.x = xy.x + 0.25 / _Width;
   xyr.y = xy.y;

   int x = ( int ) fmod( xy.x * _Width, 4 );

   if ( x == 0 )
   {
      float4 src = tex2D( InputSamplerPoint, xyr );

      ret.b = src.r;
      ret.g = src.g;
      ret.r = src.b;
   }
   else if ( x == 1 )
   {
      float4 srcl = tex2D( InputSamplerPoint, xyl );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = srcr.g;
      ret.g = srcr.b;
      ret.r = srcl.a;
   }
   else if ( x == 2 )
   {
      float4 srcl = tex2D( InputSamplerPoint, xyl );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = srcr.b;
      ret.g = srcl.a;
      ret.r = srcl.r;
   }
   else if ( x == 3 )
   {
      float4 src = tex2D( InputSamplerPoint, xyr );

      ret.b = src.a;
      ret.g = src.r;
      ret.r = src.g;
   }

   ret.a = 1.;

   return ret;
}

float4 RGBtoARGB_main( float2 xy : TEXCOORD1, float2 xyl : TEXCOORD2 ) : COLOR
{
   float4 ret;

   float2 xyr;

   // shift a quarter pixel right
   xyr.x = xy.x + 0.25 / _Width;
   xyr.y = xy.y;

   int x = ( int ) fmod( xy.x * _Width, 4 );

   if ( x == 0 )
   {
      float4 src = tex2D( InputSamplerPoint, xyr );

      ret.b = src.b;
      ret.g = src.g;
      ret.r = src.r;
   }
   else if ( x == 1 )
   {
      float4 srcl = tex2D( InputSamplerPoint, xyl );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = srcl.a;
      ret.g = srcr.b;
      ret.r = srcr.g;
   }
   else if ( x == 2 )
   {
      float4 srcl = tex2D( InputSamplerPoint, xyl );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = srcl.r;
      ret.g = srcl.a;
      ret.r = srcr.b;
   }
   else if ( x == 3 )
   {
      float4 src = tex2D( InputSamplerPoint, xyr );

      ret.b = src.g;
      ret.g = src.r;
      ret.r = src.a;
   }

   ret.a = 1.;

   return ret;
}

float4 RGBtoARGBFlip_main( float2 xy : TEXCOORD1, float2 xyl : TEXCOORD2 ) : COLOR
{
   float4 ret;

   float2 xyr;

   // shift a quarter pixel right
   xyr.x = xy.x + 0.25 / _Width;

   // flip
   xyr.y = 1 - xy.y;
   xyl.y = 1 - xyl.y;

   int x = ( int ) fmod( xy.x * _Width, 4 );

   if ( x == 0 )
   {
      float4 src = tex2D( InputSamplerPoint, xyr );

      ret.b = src.b;
      ret.g = src.g;
      ret.r = src.r;
   }
   else if ( x == 1 )
   {
      float4 srcl = tex2D( InputSamplerPoint, xyl );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = srcl.a;
      ret.g = srcr.b;
      ret.r = srcr.g;
   }
   else if ( x == 2 )
   {
      float4 srcl = tex2D( InputSamplerPoint, xyl );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = srcl.r;
      ret.g = srcl.a;
      ret.r = srcr.b;
   }
   else if ( x == 3 )
   {
      float4 src = tex2D( InputSamplerPoint, xyr );

      ret.b = src.g;
      ret.g = src.r;
      ret.r = src.a;
   }

   ret.a = 1.;

   return ret;
}
float4 RGBPlanartoARGB_main( float2 xy : TEXCOORD1, float2 xyl : TEXCOORD2 ) : COLOR
{
   float4 ret;

   float2 xyr;

   // shift a quarter pixel right
   xyr.x = xy.x + 0.25 / _Width;
   xyr.y = xy.y;

   int x = ( int ) fmod( xy.x * _Width, 4 );

   if ( x == 0 )
   {
      float4 src = tex2D( InputSamplerPoint, xyr );

      ret.b = src.b;
      ret.g = src.g;
      ret.r = src.r;
   }
   else if ( x == 1 )
   {
      float4 srcl = tex2D( InputSamplerPoint, xyl );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = srcl.a;
      ret.g = srcr.b;
      ret.r = srcr.g;
   }
   else if ( x == 2 )
   {
      float4 srcl = tex2D( InputSamplerPoint, xyl );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = srcl.r;
      ret.g = srcl.a;
      ret.r = srcr.b;
   }
   else if ( x == 3 )
   {
      float4 src = tex2D( InputSamplerPoint, xyr );

      ret.b = src.g;
      ret.g = src.r;
      ret.r = src.a;
   }

   ret.a = 1.;

   return ret;
}

float4 ChannelExpand_main( float2 xy : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSamplerPoint, xy );

   float f = floor( ( frac( ( xy.x * _Width ) / 4.0 ) * 4.0 ) + 0.5 );

   float4 ret = 0;

   if ( f == 0 )
      ret = src.b;
   else if ( f == 1 )
      ret = src.g;
   else if ( f == 2 )
      ret = src.r;
   else
      ret = src.a;

   ret.a = 1.0;

   return ret;
}

//--------------------------------------------------------------//
// Techniques
//--------------------------------------------------------------//
technique Null                                 { pass P1 { PixelShader = compile PROFILE null_main(); } }
technique ReverseComponentOrder                { pass P1 { PixelShader = compile PROFILE revComp_main(); } }
technique FlipVertical                         { pass P1 { PixelShader = compile PROFILE flip_main(); } }
technique ReverseComponentOrderAndFlipVertical { pass P1 { PixelShader = compile PROFILE revAndFlip_main(); } }
technique UYVYtoYUYV                           { pass P1 { PixelShader = compile PROFILE UYVYtoYUYV_main(); } }
technique YVYUtoYUYV                           { pass P1 { PixelShader = compile PROFILE YVYUtoYUYV_main(); } }
technique YV12toYUYV                           { pass P1 { PixelShader = compile PROFILE YV12toYUYV_main(); } }
technique RotateLeft                           { pass P1 { PixelShader = compile PROFILE RotateLeft_main(); } }
technique Rotateright                          { pass P1 { PixelShader = compile PROFILE RotateRight_main(); } }
technique RGBtoARGB                            { pass P1 { PixelShader = compile PROFILE RGBtoARGB_main(); } }
technique BGRtoARGB                            { pass P1 { PixelShader = compile PROFILE BGRtoARGB_main(); } }
technique RGBtoARGB_FlipVertical               { pass P1 { PixelShader = compile PROFILE RGBtoARGBFlip_main(); } }
technique RGBPlanartoARGB                      { pass P1 { PixelShader = compile PROFILE RGBPlanartoARGB_main(); } }
technique ChannelExpand                        { pass P1 { PixelShader = compile PROFILE ChannelExpand_main(); } }
